@@ -24,11 +24,14 @@ except ImportError:
   from elementtree import ElementTree
 import gdata.service
 import gdata
+import gdata.auth
 import atom
 import atom.service
+import atom.token_store
 import gdata.base
 import os.path
 from gdata import test_data
+import atom.mock_http
 
 
 username = ''
@@ -190,15 +193,19 @@ class GDataServiceUnitTest(unittest.TestCase):
   def testCorrectLogin(self):
     try:
       self.gd_client.ProgrammaticLogin()
-      self.assert_(self.gd_client.auth_token is not None)
+      self.assert_(isinstance(
+          self.gd_client.token_store.find_token(
+              'http://base.google.com/base/feeds/'),
+          gdata.auth.ClientLoginToken))
       self.assert_(self.gd_client.captcha_token is None)
       self.assert_(self.gd_client.captcha_url is None)
     except gdata.service.CaptchaRequired:
-    
       self.fail('Required Captcha')
 
-  def testDefaultHandler(self):
-    self.assertEquals(self.gd_client.handler, atom.service)
+  def testDefaultHttpClient(self):
+    self.assert_(isinstance(self.gd_client.http_client, 
+                            atom.http.HttpClient))
+
 
   def testGet(self):
     try:
@@ -355,6 +362,31 @@ class GDataServiceUnitTest(unittest.TestCase):
     # Delete the item the test just created.
     response = self.gd_client.Delete(response.id.text)
     self.assert_(response)
+
+  def testCaptchaUrlGeneration(self):
+    # Populate the mock server with a pairing for a ClientLogin request to a
+    # CAPTCHA challenge.
+    mock_client = atom.mock_http.MockHttpClient()
+    captcha_response = atom.mock_http.MockResponse(
+        body="""Url=http://www.google.com/login/captcha
+Error=CaptchaRequired
+CaptchaToken=DQAAAGgAdkI1LK9
+CaptchaUrl=Captcha?ctoken=HiteT4b0Bk5Xg18_AcVoP6-yFkHPibe7O9EqxeiI7lUSN
+""", status=403, reason='Access Forbidden')
+    mock_client.add_response(captcha_response, 'POST', 
+        'https://www.google.com/accounts/ClientLogin')
+
+    # Set the exising client's handler so that it will make requests to the
+    # mock service instead of the real server.
+    self.gd_client.http_client = mock_client
+
+    try:
+      self.gd_client.ProgrammaticLogin()
+      self.fail('Login attempt should have caused a CAPTCHA challenge.')
+    except gdata.service.CaptchaRequired, error:
+      self.assertEquals(self.gd_client.captcha_url, 
+          ('https://www.google.com/accounts/Captcha?ctoken=HiteT4b0Bk5Xg18_'
+           'AcVoP6-yFkHPibe7O9EqxeiI7lUSN'))
       
 
 class QueryTest(unittest.TestCase):
@@ -417,6 +449,14 @@ class QueryTest(unittest.TestCase):
     self.assertEquals(self.query['max-results'], '10')
     self.assertEquals(self.query['start-index'], '5')
 
+  def testPassInCategoryListToConstructor(self):
+    query = gdata.service.Query(feed='/feed/sample', categories=['foo', 'bar',
+            'eggs|spam'])
+    url = query.ToUri()
+    self.assert_(url.find('/foo') > -1)
+    self.assert_(url.find('/bar') > -1)
+    self.assert_(url.find('/eggs%7Cspam') > -1)
+
 
 class GetNextPageInFeedTest(unittest.TestCase):
 
@@ -436,9 +476,72 @@ class GetNextPageInFeedTest(unittest.TestCase):
     self.assert_(feed2.__class__ == feed.__class__)
 
 
+class ScopeLookupTest(unittest.TestCase):
+
+  def testLookupScopes(self):
+    scopes = gdata.service.lookup_scopes('cl')
+    self.assertEquals(scopes, gdata.service.CLIENT_LOGIN_SCOPES['cl'])
+    scopes = gdata.service.lookup_scopes(None)
+    self.assert_(scopes is None)
+    scopes = gdata.service.lookup_scopes('UNKNOWN_SERVICE')
+    self.assert_(scopes is None)
+
+
+class TokenLookupTest(unittest.TestCase):
+
+  def setUp(self):
+    self.client = gdata.service.GDataService()
+
+  def testSetAndGetClientLoginTokenWithNoService(self):
+    self.assert_(self.client.auth_token is None)
+    self.client.SetClientLoginToken('foo')
+    self.assert_(self.client.auth_token is None)
+    self.assert_(self.client.token_store.find_token(
+        atom.token_store.SCOPE_ALL) is not None)
+    self.assertEquals(self.client.GetClientLoginToken(), 'foo')
+    self.client.SetClientLoginToken('foo2')
+    self.assertEquals(self.client.GetClientLoginToken(), 'foo2')
+
+  def testSetAndGetClientLoginTokenWithService(self):
+    self.client.service = 'cp'
+    self.client.SetClientLoginToken('bar')
+    self.assertEquals(self.client.GetClientLoginToken(), 'bar')
+    # Changing the service should cause the token to no longer be found.
+    self.client.service = 'gbase'
+    self.assert_(self.client.GetClientLoginToken() is None)
+
+  def testSetAndGetClientLoginTokenWithScopes(self):
+    scopes = gdata.service.CLIENT_LOGIN_SCOPES['cl'][:]
+    scopes.extend(gdata.service.CLIENT_LOGIN_SCOPES['gbase'])
+    self.client.SetClientLoginToken('baz', scopes=scopes)
+    self.assert_(self.client.GetClientLoginToken() is None)
+    self.client.service = 'cl'
+    self.assertEquals(self.client.GetClientLoginToken(), 'baz')
+    self.client.service = 'gbase'
+    self.assertEquals(self.client.GetClientLoginToken(), 'baz')
+    self.client.service = 'wise'
+    self.assert_(self.client.GetClientLoginToken() is None)
+
+  def testLookupUsingTokenStore(self):
+    scopes = gdata.service.CLIENT_LOGIN_SCOPES['cl'][:]
+    scopes.extend(gdata.service.CLIENT_LOGIN_SCOPES['gbase'])
+    self.client.SetClientLoginToken('baz', scopes=scopes)
+    token = self.client.token_store.find_token(
+        'http://www.google.com/calendar/feeds/foo')
+    self.assertEquals(token.get_token_string(), 'baz')
+    self.assertEquals(token.auth_header, '%s%s' % (
+        gdata.auth.PROGRAMMATIC_AUTH_LABEL, 'baz'))
+    token = self.client.token_store.find_token(
+        'http://www.google.com/calendar/')
+    self.assert_(isinstance(token, gdata.auth.ClientLoginToken) == False)
+    token = self.client.token_store.find_token(
+        'http://www.google.com/base/feeds/snippets')
+    self.assertEquals(token.get_token_string(), 'baz')
+
+
 if __name__ == '__main__':
-  print ('NOTE: Please run these tests only with a test account. ' +
-      'The tests may delete or update your data.')
+  print ('GData Service Media Unit Tests\nNOTE: Please run these tests only '
+         'with a test  account. The tests may delete or update your data.')
   username = raw_input('Please enter your username: ')
   password = getpass.getpass()
   unittest.main()
